#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include "image.h"


int window_width  = 512;
int window_height = 512;

float theta = 0.0;

// texture id
GLuint texid = 0;

// file storing our texture data - must be a power of 2 by a power of 2 
// in size, such as 64x64 or 128x256.  100x100 is invalid.
const char *texture_filename = "./texture.bmp";


void display () 
{
  GLenum errCode = glGetError();
  if (errCode != GL_NO_ERROR) 
  {
      const GLubyte* errString = gluErrorString(errCode);
      fprintf( stderr, "OpenGL error: %s\n", errString );
  }

  //clear the screen at the begin of drawing each frame.
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  glPushMatrix();

  // set polygon colors to white
  glColor3f( 1, 1, 1 );

  // select our texture handle and enable texturing
  glBindTexture( GL_TEXTURE_2D, texid );
  glEnable( GL_TEXTURE_2D );

  // draw a quad with texture coordinates specified at each vertex
  glBegin( GL_QUADS );
    glTexCoord2f( 0, 0 ); glVertex2f( 0, 0 );
    glTexCoord2f( 1, 0 ); glVertex2f(  50, 0 );
    glTexCoord2f( 1, 1 ); glVertex2f(  50,  50 );
    glTexCoord2f( 0, 1 ); glVertex2f( 0,  50 );
  glEnd();

  // disable texturing to render normally
  glBindTexture( GL_TEXTURE_2D, 0 );
  glDisable( GL_TEXTURE_2D );


  // draw another quad, with texture coordinates
  glBegin( GL_QUADS );
    glTexCoord2f( 0, 0 ); glVertex2f( 100, 100);
    glTexCoord2f( 1, 0 ); glVertex2f(  120, 100);
    glTexCoord2f( 1, 1 ); glVertex2f(  120,  120);
    glTexCoord2f( 0, 1 ); glVertex2f( 100,  120);
  glEnd();

  glPopMatrix();

  glutSwapBuffers();  
  glutPostRedisplay();

  // increment angle for animation
  theta += 0.1;
}


void reshape ( int width, int height ) 
{
  window_width  = width;
  window_height = height; 

  glViewport( 0, 0, width, height );  
}


void keyboard_func ( unsigned char key, int x, int y )
{
  switch( key )
  {
  case 'q':
  case 'Q':
    exit( 0 );
    break;

  default:
    break;
  }
}

//call back function for mouse click event
void mouse_click_func ( int button, int state, int x, int y )
{
	//example
	if (button==GLUT_LEFT_BUTTON && state==GLUT_DOWN)
		printf("Left click at coordinate %d, %d.\n",x,y);

}

//call back funciton for mouse move
//you can get the current position of mouse from x,y
void mouse_move_func ( int x, int y )
{

}


void idle_func ()
{
}

//load textures once at the initializing stage
void initTexture ()
{
  // read in texture image
  Image *image = new Image();
  image->read( texture_filename );
  if( ! image->good() )
  {
    fprintf( stderr, "ERROR: couldn't load %s\n", texture_filename );

	fgetc( stdin );
    exit( 1 );
  }

  // generate texture handle
  glGenTextures( 1, &texid );

  // set current texture to our handle
  glBindTexture( GL_TEXTURE_2D, texid );

  // set the magnification / minification filter to linear sampling
  // (instead of GL_NEAREST for point sampling)
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

  // set the texture coordinate wrap mode to repeat (instead of GL_CLAMP)
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );

  // pixels will be packed on the byte
  glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );

  // load image data into texture object
  glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB,
                image->getWidth(), image->getHeight(), 0,
                GL_RGB, GL_UNSIGNED_BYTE, image->getPixels() );

  // set current texture to none
  glBindTexture( GL_TEXTURE_2D, 0 );

  // free the image memory - no need to store it anymore
  delete image;
}


void init ()
{
  glClearColor( 0.0, 0.0, 0.0, 0.0 );
  glShadeModel( GL_SMOOTH );
  glEnable( GL_DEPTH_TEST );

  //register call back functions
  glutDisplayFunc( display );
  glutReshapeFunc( reshape );
  glutKeyboardFunc( keyboard_func );
  glutMouseFunc( mouse_click_func );
  glutMotionFunc( mouse_move_func );
  glutIdleFunc( idle_func );

  initTexture();

  //set projection matrix
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluOrtho2D(0,window_width,0,window_height);

  //set modelview matrix
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
}


int main ( int argc, char **argv )
{
  glutInit( &argc, &argv[0] ); 
  glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
  glutInitWindowPosition( 100, 100 );
  glutInitWindowSize( window_width, window_height );
  glutCreateWindow( "template" );
 
  init();

  glutMainLoop();//start the rendering loop

  return 0;
}
